@@ -13,7 +13,7 @@ use Scalar::Util qw(looks_like_number blessed reftype refaddr);
 our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
 our $DATE = '2024-03-12'; # DATE
 our $DIST = 'Data-Dump-HTML-Collapsible'; # DIST
-our $VERSION = '0.002'; # VERSION
+our $VERSION = '0.003'; # VERSION
 
 our @EXPORT = qw(dd);
 our @EXPORT_OK = qw(dump);
@@ -102,7 +102,7 @@ sub _quote_key {
 }
 
 sub _dump {
-    my ($val, $subscript) = @_;
+    my ($val, $subscript, $depth) = @_;
 
     my $ref = ref($val);
     if ($ref eq '') {
@@ -123,13 +123,13 @@ sub _dump {
             return _double_quote($val);
         }
     }
-    my $refaddr = refaddr($val);
+    my $refaddr = sprintf("%x", refaddr($val));
     $_subscripts{$refaddr} //= $subscript;
     if ($_seen_refaddrs{$refaddr}++) {
         my $target = "\$var" .
             ($_subscripts{$refaddr} ? "->$_subscripts{$refaddr}" : "");
-        push @_fixups, "\$var->$subscript=$target;";
-        return _single_quote($target);
+        push @_fixups, "\$var->$subscript = $target;\n";
+        return "<a href=#r$refaddr>"._single_quote($target)."</a>";
     }
 
     my $class;
@@ -145,37 +145,38 @@ sub _dump {
         $ref = reftype($val);
     }
 
-    my $res;
+    my $res = "";
+    $res .= "<a name=r$refaddr></a>";
     if ($ref eq 'ARRAY') {
-        $res = "<details><summary>$subscript ".encode_entities("$val")."</summary>[";
+        $res .= "<details><summary># ARRAY(0x$refaddr)</summary>[";
         my $i = 0;
         for (@$val) {
-            $res .= ",\n" if $i;
-            $res .= _dump($_, "$subscript\[$i]");
+            $res .= ",   # ".("." x $depth)."[".($i-1)."]\n" if $i;
+            $res .= _dump($_, "$subscript\[$i]", $depth+1);
             $i++;
         }
         $res .= "]</details>";
     } elsif ($ref eq 'HASH') {
-        $res = "<details><summary>$subscript ".encode_entities("$val")."</summary>{";
+        $res .= "<details><summary># HASH(0x$refaddr)</summary>{";
         my $i = 0;
         for (sort keys %$val) {
-            $res .= ",\n" if $i;
+            $res .= ",   # ".("." x $depth)."{".($i-1)."}\n" if $i;
             my $k = _quote_key($_);
-            my $v = _dump($val->{$_}, "$subscript\{$k}");
+            my $v = _dump($val->{$_}, "$subscript\{$k}", $depth+1);
             $res .= "$k =&gt; $v";
             $i++;
         }
         $res .= "}</details>";
     } elsif ($ref eq 'SCALAR') {
         if (defined $class) {
-            $res = "do { my \$o="._dump($$val, $subscript)."; \\\$o}";
+            $res .= "do { my \$o="._dump($$val, $subscript)."; \\\$o}";
         } else {
-            $res = "\\"._dump($$val, $subscript);
+            $res .= "\\"._dump($$val, $subscript);
         }
     } elsif ($ref eq 'REF') {
-        $res = "\\"._dump($$val, $subscript);
+        $res .= "\\"._dump($$val, $subscript);
     } elsif ($ref eq 'CODE') {
-        $res = $OPT_DEPARSE ? _dump_code($val) : 'sub{"DUMMY"}';
+        $res .= $OPT_DEPARSE ? _dump_code($val) : 'sub{"DUMMY"}';
     } else {
         die "Sorry, I can't dump $val (ref=$ref) yet";
     }
@@ -193,12 +194,12 @@ sub _dd_or_dump {
 
     my $res;
     if (@_ > 1) {
-        $res = "(" . join(",", map {_dump($_, '')} @_) . ")";
+        $res = "(" . join(",\n", map {_dump($_, '', 0)} @_) . ")";
     } else {
-        $res = _dump($_[0], '');
+        $res = _dump($_[0], '', 0);
     }
     if (@_fixups) {
-        $res = "do{my\$var=$res;" . join("", @_fixups) . "\$var}";
+        $res = "do { my \$var = $res;\n" . join("", @_fixups) . "\$var }";
     }
 
     $res = "<style>details {  margin-left: 1em; } summary { margin-left: -1em; }</style><pre>$res</pre>";
@@ -228,7 +229,7 @@ Data::Dump::HTML::Collapsible - Dump Perl data structures as HTML document with
 
 =head1 VERSION
 
-This document describes version 0.002 of Data::Dump::HTML::Collapsible (from Perl distribution Data-Dump-HTML-Collapsible), released on 2024-03-12.
+This document describes version 0.003 of Data::Dump::HTML::Collapsible (from Perl distribution Data-Dump-HTML-Collapsible), released on 2024-03-12.
 
 =head1 SYNOPSIS
 
@@ -237,9 +238,13 @@ This document describes version 0.002 of Data::Dump::HTML::Collapsible (from Per
 
 =head1 DESCRIPTION
 
-This module dumps Perl data structure to HTML document where you can expand and
-collapse nodes to drill down and roll up your data. It currently uses the C<<
-<summary> >> and C<< <details> >> HTML elements.
+This module is a L<Data::Dump> variant that dumps Perl data structure to HTML
+document where you can expand and collapse nodes to drill down and roll up your
+data. It currently uses the C<< <summary> >> and C<< <details> >> HTML elements.
+
+There are other collapsible HTML dumper modules on CPN (see L</SEE ALSO>). This
+module tries to make its text output valid Perl while adding some visual cues
+to let human reader track the nodes more easily.
 
 =head1 VARIABLES
 
@@ -314,14 +319,14 @@ Source repository is at L<https://github.com/perlancar/perl-Data-Dump-HTML-Colla
 =head1 SEE ALSO
 
 Other data structure dumpers to (collapsible) tree: L<Data::HTML::TreeDumper>
-(doesn't handle recursion), L<Data::TreeDumper>
-(L<Data::TreeDumper::Renderer::DHTML>, L<Data::TreeDumper::Renderer::GTK>),
-L<Data::Dumper::GUI>.
+(also uses C<< <details> >> and C<< <summary> >> HTML elements, doesn't handle
+recursion), L<Data::TreeDumper> (L<Data::TreeDumper::Renderer::DHTML>,
+L<Data::TreeDumper::Renderer::GTK>), L<Data::Dumper::GUI>.
 
 Other data structure dumpers that outputs to HTML: L<Data::HTMLDumper>,
 L<Data::Dumper::HTML>, L<Data::Format::Pretty::HTML>.
 
-Other data structure dumpers: L<Data::Dumper>, L<Data::Dump>,
+Other data structure dumpers, among others: L<Data::Dumper>, L<Data::Dump>,
 L<Data::Dump::Color>, L<Data::Dmp>, L<Data::Printer>.
 
 =head1 AUTHOR
